-----------------------------------------------------------------------------
-- This module contains the PCIBUS3 arbiter
--
-- There are four potential masters on this bus:
-- PCI-PCI bridge which has device number 0
-- logic board which has device number 1
-- Universe II which has device number 2
-- buffer control board which has device number 3
-- 
-- The algorithm gives Universe II the highest priority
-- Once request from Universe II is detected, grant will
-- be withdrawn from the current master. All other
-- masters share the bus based a round robin priority scheme,
-- it rotates in the order 3,1,0,3....
-- Bus is always parked on the last master using the bus.
-- After reset, bus is parked on Universe II 
-- Oct.16.2000	S.X.Wu
--library altera;
--use altera.maxplus2.all;
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
---------------------------------------------------------------------------
entity arbiter is

        port ( clock:		in std_logic;  -- PCI clock input, rising edge active
               rst_n:		in std_logic;  --- PCI reset input 
               frame_n:		in std_logic;  --- PCI frame inut
               irdy_n:		in std_logic;  --- PCI irdy inut
               en_master:	in std_logic_vector(1 downto 0);  --- enable master request
               req_in_n:	in std_logic_vector(3 downto 0);  --- PCI bus request input from the PCI-PCI bridge
-- outputs following
               rqst_mask_o: out std_logic_vector(1 downto 0);
               gnt_pci_n:	out std_logic_vector(3 downto 0)  -- for PCI outputs
              );
end entity arbiter;

----------------------------Architecture ---------------------------

architecture logic_design of arbiter is

---------------------Signal and constant Declaration--------------------------
signal		vme_access:		std_logic;
signal		vme_access_dl:	std_logic;
signal		delay_cnt:		std_logic_vector(1 downto 0); -- grant signals for internal use
signal		idle_cnt:		unsigned(3 downto 0); -- grant signals for internal use
signal		rqst_mask:		std_logic_vector(1 downto 0); -- grant signals for internal use
signal		gnt_n:			std_logic_vector(3 downto 0); -- grant signals for internal use
signal		req_n:			std_logic_vector(3 downto 0); -- PCI request synchronized with PCI clock
signal		m0,m1,dm0,dm1:	std_logic;  --- registers to store bus master infomation
signal		bus_available:	std_logic;  --- bus idle and round robin devices request active
signal      pm_0:			std_logic;  --- previous PCI master was PCI-PCI bridge
signal      pm_1:			std_logic;  --- previous PCI master was logic board
signal      pm_3:			std_logic;  --- previous PCI master was buffer control board
signal      gnt_active:		std_logic;  --- bus is granted to someone
signal      rr_dev_took_bus:std_logic;  --- devices 0, 1 or 3 used the bus
signal		req2_n:			std_logic;  --- this is an OR of Universe II request and rst_n rising edge condition
signal		sortie_UV2_n:	std_logic;  --- this signal takes grant parked on Universe away, low active.
signal		vcc:			std_logic;  --- 

-- component declaration
component channel_gnt

        port ( clock:		in std_logic;  -- PCI clock input, rising edge active
               rst_n:		in std_logic;  --- PCI reset input 
               req_a_n:		in std_logic;  --- PCI bus request input from the same channel as the grant output
               req_b_n:		in std_logic;  --- PCI bus request input with priority higher than req_c_n
               req_c_n:		in std_logic;  --- PCI bus request input 
               req_vme_n:	in std_logic;  --- PCI bus request input from Universe II 
               pm_b:		in std_logic;  --- previous PCI master which gives req_b_n higher priority over req_a_n
               pm_c:		in std_logic;  --- previous PCI master which gives req_a_n the lowest priority
               gnt_active:	in std_logic;  --- bus is granted to someone
-- outputs following
               gnt_a_n:		out std_logic; -- for inernal use only
               gnt_a_pci_n:	out std_logic  -- for PCI outputs
              );
end component;

begin
rqst_mask_o <= rqst_mask;
process(clock,rst_n)
 begin
   if (rst_n = '0') then
     vme_access <= '0';
     vme_access_dl <= '0';
     delay_cnt <= "00";
     idle_cnt <= "0000";
     rqst_mask <= "00";
   elsif (clock'event and clock = '1')then
     vme_access <= not frame_n and not gnt_n(2);
     vme_access_dl <= vme_access;
     if(vme_access = '1' and vme_access_dl = '0')then
       delay_cnt(1) <= delay_cnt(1) or delay_cnt(0);
       delay_cnt(0) <= not delay_cnt(0) or delay_cnt(1);
     end if;
     if(frame_n = '1' and delay_cnt = "11" and ((req_in_n(1) = '0' and gnt_n(1) = '0') or (req_in_n(3) = '0' and gnt_n(3) = '0')))then
       idle_cnt <= idle_cnt + 1;
       if(idle_cnt = 15)then
         rqst_mask <= not gnt_n(3) & not gnt_n(1);
       end if;
     else
       idle_cnt <= "0000";
     end if; 
   else
   end if;
end process;
--gnt_pci_n(3) <= gnt_n(3) when rqst_mask(1) = '0' else 'Z';
--gnt_pci_n(1) <= gnt_n(1) when rqst_mask(0) = '0' else 'Z';
gnt_pci_n(3) <= gnt_n(3);
gnt_pci_n(1) <= gnt_n(1);
grant0:	channel_gnt
    port map (
    		clock		=> clock,
    		rst_n		=> rst_n,
    		req_a_n		=> req_n(0),
    		req_b_n		=> req_n(1),
    		req_c_n		=> req_n(3),
--    		req_vme_n	=> req_n(2),
    		req_vme_n	=> req2_n,
    		pm_b		=> pm_3,
    		pm_c		=> pm_0,
    		gnt_active	=> gnt_active,
    		gnt_a_n		=> gnt_n(0),
    		gnt_a_pci_n	=> gnt_pci_n(0));
    		
grant1:	channel_gnt
    port map (
    		clock		=> clock,
    		rst_n		=> rst_n,
    		req_a_n		=> req_n(1),
    		req_b_n		=> req_n(3),
    		req_c_n		=> req_n(0),
--    		req_vme_n	=> req_n(2),
    		req_vme_n	=> req2_n,
    		pm_b		=> pm_0,
    		pm_c		=> pm_1,
    		gnt_active	=> gnt_active,
    		gnt_a_n		=> gnt_n(1));
--    		gnt_a_pci_n	=> gnt_pci_n(1));

-- Universe II has the highest priority in getting bus. At power up,     		
grant2:	channel_gnt
    port map (
    		clock		=> clock,
    		rst_n		=> rst_n,
    		req_a_n		=> req2_n, -- this is not req_n(2)!!!
    		req_b_n		=> vcc,
    		req_c_n		=> vcc,
    		req_vme_n	=> sortie_UV2_n, -- this is how to exit bus parked on Universe II
    		pm_b		=> vcc,
    		pm_c		=> vcc,
    		gnt_active	=> gnt_active,
    		gnt_a_n		=> gnt_n(2),
    		gnt_a_pci_n	=> gnt_pci_n(2));
    		
grant3:	channel_gnt
    port map (
    		clock		=> clock,
    		rst_n		=> rst_n,
    		req_a_n		=> req_n(3),
    		req_b_n		=> req_n(0),
    		req_c_n		=> req_n(1),
--    		req_vme_n	=> req_n(2),
    		req_vme_n	=> req2_n,
    		pm_b		=> pm_1,
    		pm_c		=> pm_3,
    		gnt_active	=> gnt_active,
    		gnt_a_n		=> gnt_n(3));
--    		gnt_a_pci_n	=> gnt_pci_n(3));

-- process to synchronize the PCI request inputs
process(clock,rst_n)
 begin
   if (rst_n = '0') then
     req_n <= "1111";
   elsif (clock'event and clock = '1')then
     req_n(0) <= req_in_n(0);
     req_n(2) <= req_in_n(2);
--     req_n(1) <= req_in_n(1) or not en_master(0);
--     req_n(3) <= req_in_n(3) or not en_master(1);
     req_n(1) <= req_in_n(1) or rqst_mask(0);
     req_n(3) <= req_in_n(3) or rqst_mask(1);
   else
   end if;
end process;
-- previous master generation    		
process(clock,rst_n)
 begin
--  if PCI reset active, previous master defaulted to PCI-PCI bridge
   if (rst_n = '0')then
     dm0 <= '0';
     dm1 <= '0';
     m0 <= '0';
     m1 <= '0';
     bus_available <= '0';
   else
     if( clock'event and clock = '1')then
       dm0 <= not gnt_n(1) or not gnt_n(3);
       dm1 <= not gnt_n(3);
       bus_available <= frame_n and irdy_n and ( not gnt_n(0) or not gnt_n(1) or not gnt_n(3));
       if( rr_dev_took_bus = '1')then
         m0 <= dm0; 
         m1 <= dm1;
       end if;   
     end if;
   end if;
end process;
-- if there is no request and no grant active, i.e., after a PCI reset, grant is given to Universe II
--   req2_n <= req_n(2) and not( not gnt_active and req_n(0) and req_n(1) and req_n(3));
-- this causes always park to UniverseII
   req2_n <= req_n(2) and not(gnt_n(2) and req_n(0) and req_n(1) and req_n(3));
-- if bus is parked on Universe II and others want to get access, this signal does the work
   sortie_UV2_n <= not req_n(2) or ( req_n(0) and req_n(1) and req_n(3));
-- decoding the previous master signals
   pm_0 <= not m0 and not m1;
   pm_1 <= m0 and not m1;
   pm_3 <= m0 and m1;
   rr_dev_took_bus <= bus_available and not frame_n;
-- grant is active if any grant is low
   gnt_active <= not (gnt_n(0) and gnt_n(1) and gnt_n(2) and gnt_n(3));
-- port map does not accept constant   
   vcc <= '1';
end architecture logic_design;   --End of architecture block
