-----------------------------------------------------------------------------
-- This module contains the PCIBUS3 arbiter's grant signal generation for
-- an individual channel
--
-- There are four potential masters on this bus:
-- PCI-PCI bridge which has device number 0
-- logic board which has device number 1
-- Universe II which has device number 2
-- buffer control board which has device number 3
-- 
-- The algorithm gives Universe II the highest priority
-- Once request from Universe II is detected, grant will
-- be withdrawn from the current master. All other
-- masters share the bus based a round robin priority scheme,
-- it rotates in the order 3,1,0,3....
-- Bus is always parked on the last master using the bus.
-- After reset, bus is parked on Universe II 
-- Oct.16.2000	S.X.Wu
--library altera;
--use altera.maxplus2.all;
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
---------------------------------------------------------------------------
entity channel_gnt is

        port ( clock:		in std_logic;  -- PCI clock input, rising edge active
               rst_n:		in std_logic;  --- PCI reset input 
               req_a_n:		in std_logic;  --- PCI bus request input from the same channel as the grant output
               req_b_n:		in std_logic;  --- PCI bus request input with priority higher than req_c_n
               req_c_n:		in std_logic;  --- PCI bus request input 
               req_vme_n:	in std_logic;  --- PCI bus request input from Universe II 
               pm_b:		in std_logic;  --- previous PCI master which gives req_b_n higher priority over req_a_n
               pm_c:		in std_logic;  --- previous PCI master which gives req_a_n the lowest priority
               gnt_active:	in std_logic;  --- bus is granted to someone
-- outputs following
               gnt_a_n:		out std_logic; -- for inernal use only
               gnt_a_pci_n:	out std_logic  -- for PCI outputs
              );
end entity channel_gnt;

----------------------------Architecture ---------------------------

architecture logic_design of channel_gnt is

---------------------Signal and constant Declaration--------------------------
signal gnt: std_logic;
begin

--  need two states to write data
process(clock,rst_n)
 begin
--  if PCI reset active, grant must go high
   if (rst_n = '0')then
     gnt <= '1';
     gnt_a_pci_n <= '1';
     else
      if( clock'event and clock = '1')then
        if((gnt = '1' and ( gnt_active = '1' or req_a_n = '1')) or
           (pm_b = '1' and req_b_n = '0') or
           ((pm_c = '1' or req_a_n = '1') and (req_b_n = '0' or req_c_n = '0')) or
           req_vme_n = '0')then
             gnt <='1';
             gnt_a_pci_n <= '1';
        else
            gnt <='0';
            gnt_a_pci_n <= '0';
        end if;
      end if;
   end if;
end process;
-- gnt_a_n get value from dummy signal gnt
   gnt_a_n <= gnt;
end architecture logic_design;   --End of architecture block
