-----------------------------------------------------------------------------
-- This module is the local control target logic
-- all accesses are 32-bit double word access, although not all bits are always used.
---------------------------------------------------------------------------------
-- flash memory access is in memory space and controlled by BAR0
-- BAR0 occupies 16 bytes of memory space
-- offset 0x0 of BAR0 is a write only FLASH address register, only bits 18-0 are implemented
-- offset 0x4 of BAR0 is a write/read FLASH data register, only bits 7-0 are implemented
-- Only byte accesses are accepted. Erase or program the flash need special commands as specified in the 
-- datasheets of the flash memory chip (29F002NB)
-- offset 0x8 and oxc of BAR0 are not used.
----------------------------------------------------------------------------------
-- BAR1 occupies 32 bytes of io space
-- offset 0x0 of BAR1 is used for communicating with the VTM PIC micro-controller
-- The protocol is specified in the VTM specs.
-- writing to it sends bits 7-0 to PIC and the data returned from PIC can be read back about 2 milliseconds
-- later. If bit 8 of the readback data is set, PIC failed to respond. Otherwise, bit 7-0 contains the data
-- access to this location before the previvious command has finished will cause a PCI retry 
----------------------------------------------------------------------------------
-- offset 0x4 of BAR1 is used for access the P3 backplane bus controller. It is a write/read register
-- read from it returns the status of the backplane bus controller. If the controller is not configured ( or
-- has an out-of-date firmware), it reads as 0xff. Otherwise, bits 3-2 are the controller firmware revision
-- (current version is 1), and bit 1 should be '1' if the motherboard has backplane bus pullup resistor networks
-- installed. Bit 0 powers up opposite to bit 1 and must be set to equal to bit 1 to enable the controller.
-- to set bit 0, write 0x3. To reset bit 0, write 0x2.
-- offset 0x8 is master enable register. Only two bits are implmented:
-- bit 0 set to '1' to enable pci3 master.
-- bit 1 set to '1' to enable bus controller master
-- offset 0xc of BAR1 is write-only and used to start a local control chip reconfiguration
-- To start a reconfiguration, write a special data word to offset 0xc of BAR1
-- This special data word must have bit 18 = '1' and bits 15 downto 0 = "A5A5". Bits 17 and 16 specify the
-- set of configuration data to be used.
-- Each set has 64K bytes. Power-on always configures from set 0.
-- Set 3 can access the flash and is used to update other sets of configuration data. Any failed configuration
-- attempt in other sets leads automatically to a set 3 reconfiguration. Normally one should not touch set 3.
-- if set 3 data are corrupted, you have to configure from JTAG.
-- offset 0x10 of BAR1 is JTAG data register. It is a write/read register.
-- writing a 32-bit word to the JTAG data register causes N TCK being sent to the selected JTAG device together
-- with their corresponding TDI and TMS data. bits N+15 thru 16 are used as TMS data and bits N thru 0 as TDI data.
-- the MSB goes out first. TDO data are read back following each falling edge of the TCK and are read back
-- as bits N thru 0 from the JTAG data register. Ignore bits 31 thru N+1 from read back
-- offset 0x14 of BAR1 is JTAG control register. It is a write/read register.
-- bits 5-0 of the JTAG control register set the JTAG clock frequency. If bits 5-0 is set to n,
-- TCK frequency equals 16MHz/(1 + n)
-- bits 10-6 of the JTAG control register select the JTAG device:
-- 0x1 => log3		0x3 => bc		0xa => mip4		0xb => mip5		0xc => mip6
-- 0x9 => log2		0x11 => log1	0x12 => mip1	0x13 => mip2	0x14 => mip3		
-- bits 14-11 of the JTAG control register decide the number of JTAG cycles performed:
-- All zeros results in N = 0, and all ones results in N = 16
-- Jun.5.2002	S.X.Wu
-- July 29 2002 modified to add jtag_csr bits 14 thru 11
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
---------------------------------------------------------------------------
entity lctl is
        port ( 
-- inputs from PCI interface        
               clk:			in std_logic;  -- PCI clock input, rising edge active
               rstn:		in std_logic;  --- PCI reset input 
               lt_dxfrn:	in std_logic;  --- last target data transfer successful
               lt_tsr:		in std_logic_vector(11 downto 0);  --- target transaction status
               l_dato:		in std_logic_vector(31 downto 0);  --- local data input
               l_adro:		in std_logic_vector(31 downto 0);  --- local address input
               l_cmdo:		in std_logic_vector(3 downto 0);  --- local command input
-- outputs to PCI interface
               lt_rdyn:		out std_logic;  --- local target ready for r/w data
               lt_discn:	out std_logic;  --- local target disconnect output
               l_adi:		out std_logic_vector(31 downto 0);  --- local data output
-- inouts from FLASH memory
               flash_data:	inout std_logic_vector(7 downto 0);  --- flash memory data inputs
-- outputs to FLASH memory
               flash_oen:	out std_logic;  --- flash memory output enable
               flash_wen:	out std_logic;  --- flash memory write enable
               flash_cen:	out std_logic;  --- flash memory chip enable
               flash_adr:	out std_logic_vector(18 downto 0);  --- flash memory address outputs
-- reconfiguration control
               mc_spare:	out std_logic;
-- pic interface
-- serial signal to PIC12C671
               serial:		inout std_logic;
-- vtm_bus adapter XC95108 interface signals
               j3bus_ctl0:	out std_logic;
               j3bus_ctl1:	inout std_logic;
-- JTAG interface signals
       	       tdo: in STD_LOGIC_VECTOR (9 downto 0);
               tdi: out STD_LOGIC_VECTOR (9 downto 0);
               tck: out STD_LOGIC_VECTOR (9 downto 0);
               tms: out STD_LOGIC_VECTOR (9 downto 0);
-- enable master
			   rqst: in STD_LOGIC_VECTOR(3 downto 0);
			   gnt: in STD_LOGIC_VECTOR(3 downto 0);
			   rqst_mask: in STD_LOGIC_VECTOR(1 downto 0);
			   en_master: out STD_LOGIC_VECTOR(1 downto 0)
              );
end entity lctl;
----------------------------Architecture ---------------------------
architecture logic_design of lctl is
signal		reconf:		std_logic;
signal		pci_idle:	std_logic;
signal		bar1_data:	std_logic_vector(15 downto 0);
---------------------FLASH Signal and constant Declaration--------------------------
signal		count:		unsigned(3 downto 0); -- target timing counter
signal		temp:		std_logic_vector(7 downto 0);
type		state is	(S0,S1,S2,S3,S4,S5,S6,S7);
signal		S:	state;
---------------------JTAG Signal and constant Declaration--------------------------
signal		Sjtag:		std_logic;
signal		jtag_data:	std_logic_vector(31 downto 0);
signal		jtag_csr:	std_logic_vector(14 downto 0);
signal		fd:			unsigned(5 downto 0);
signal		freq_div:	unsigned(5 downto 0);
signal		tclk:		std_logic;
signal		jtag_tdo:	std_logic;
signal		jtag_done:	std_logic;
--------------------- BUS controller Signal and constant Declaration--------------------------
signal		Sbus:		std_logic;
signal		write_bus:		std_logic;
signal		bus_data:	std_logic_vector(7 downto 0);
signal		bus_done:	std_logic;
signal		bus_data_valid:	std_logic;
---------------------PIC Signal and constant Declaration--------------------------
type		statea is	(Sa0,Sa1,Sa2,Sa3);
signal		Spic:		statea;
constant	tc208:		positive := 207;
signal		diva:		integer range 0 to tc208; -- get 104*16 from pci clock
signal		cycles:		integer range 0 to 10;
signal		timeout:	std_logic;
signal		div16:		integer range 0 to 15;
signal		pic_retry:	std_logic;
signal		pic_busy:	std_logic;
signal		sample:		std_logic;
signal		ack:		std_logic;
signal		most_one:	std_logic;
signal		most_zero:	std_logic;
signal		serial_in:	std_logic_vector(3 downto 0);
signal		j3bus_ctl1p:	std_logic;
signal		t_j3bus_ctl1: 	std_logic;
signal		pic_data:	std_logic_vector(8 downto 0);
signal		pic_em:		std_logic_vector(8 downto 0);
-- enable master
signal		en_master_i:std_logic_vector(1 downto 0);
begin
en_master <= en_master_i;
mc_spare <= not reconf;
-- flash data IO pads are default to be on
flash_data <= temp when (lt_tsr(0) = '0' or l_cmdo(0) = '1') else (others => 'Z');
-- read back data
l_adi(31 downto 16) <= "0000000000000000";
l_adi(15 downto 0) <= "00000000" & flash_data when lt_tsr(0) = '1' else bar1_data;
pic_em <= timeout & pic_data(7 downto 0) when l_adro(3) = '0' else '0' & rqst(3) & rqst(1) & gnt(3) & gnt(1) & rqst_mask & en_master_i;
process(clk,l_adro)
variable sel_data: std_logic_vector(1 downto 0);
begin
  sel_data := l_adro(4) & l_adro(2);
  if(clk'event and clk = '1')then
    pci_idle <= not(lt_tsr(0) or lt_tsr(1));
    case sel_data is
      when "00" => bar1_data <= "0000000" & pic_em;
      when "01" => bar1_data <= "00000000" & bus_data;
      when "10" => bar1_data <= jtag_data(14 downto 0) & jtag_tdo;
      when others => bar1_data <= '0' & jtag_csr;
    end case;
  end if;
end process;
-- convert std_logic_vector to unsigned
g_fd: for i in fd'range generate
  fd(i) <= jtag_csr(i);
end generate;
process(clk)
begin
  if(clk'event and clk = '1')then
-- pic cycle takes 2ms, so retry is needed.
    pic_retry <= lt_tsr(1) and pic_busy and not l_adro(4) and not l_adro(3) and not l_adro(2);
-- mc_spare is used as clock signal to check reboot command in the configuration control logic
-- flash_adr(17:16) contains program ID and flash_adr(15:0) must contain "A5A5" as reboot password
-- flash_adr(18) must be '1' to distinguish itself from normal flash programming 
    reconf <= lt_tsr(1) and l_cmdo(0) and not l_adro(4) and l_adro(3) and l_adro(2);
-- input must be latched
    if(lt_dxfrn = '0' and l_cmdo(0) = '1')then
      if((lt_tsr(0) = '1' and l_adro(2) = '0') or reconf = '1')then
        flash_adr <= l_dato(18 downto 0);
      else
        temp <= l_dato(7 downto 0);
      end if;
    end if;
  end if;
end process;
-- the main state machine
process(clk,rstn)
begin 
  if(rstn = '0')then
    S <= S0;
    count <= "0000";
    flash_wen <= '1';
    bus_data_valid <= '0';
  elsif ( clk'event and clk = '1')then
    case S is
      when S0 =>
        count <= "0000";
-- read access
        if(l_cmdo(0) = '0' and lt_tsr(1 downto 0) /= "00")then
          if(lt_tsr(1) = '1' and l_adro(4 downto 2) = "001" and bus_data_valid = '0')then
            bus_data_valid <= '1';
            S <= S6;
          else
            S <= S1;
          end if;
-- flash write access
        elsif(lt_tsr(0) = '1')then
	        S <= S2;
        elsif(lt_tsr(1) = '1')then
          if(l_adro(4) = '1')then
-- JTAG access
	        S <= S4;
		    count(3) <= not jtag_csr(14);
		    count(2) <= not jtag_csr(13);
		    count(1) <= not jtag_csr(12);
		    count(0) <= not jtag_csr(11);
          elsif(l_adro(3) = '1')then
-- reconfigure or write master enable
	        S <= S2;
          elsif(l_adro(2) = '0')then
-- pic access
	        S <= S5;
          else
-- backplane bus controller access
	        S <= S6;
	      end if;
        else
-- default stay at S0
          S <= S0;
        end if;
        flash_wen <= '1';
-- read
      when S1 =>
        if(count /= 3)then
	      count <= count + 1;
        end if;
        if(lt_dxfrn = '0' or pic_retry = '1')then
          S <= S7;
        else
          S <= S1;
        end if;
-- flash write phase 1
      when S2 =>
        if(lt_dxfrn = '0')then
          if(l_adro(3 downto 2) = "01") then
-- write flash data
            S <= S3;
          elsif(l_adro(3 downto 2) = "10") then
            en_master_i <= l_dato(1 downto 0);
            S <= S7;
          else
-- reconfigure or flash address setting
            S <= S7;
          end if;
        else
	      S <= S2;
	    end if;
-- FLASH write phase 2
      when S3 =>
        flash_wen <= count(1);
        if(count /= 3)then
	      count <= count + 1;
          S <= S3;
        else
          S <= S7;
        end if;
      when S4 =>
-- JTAG access
	    if(freq_div = fd and tclk = '1')then
	      count <= count + 1;
	    end if;
        if(jtag_done = '1')then
          S <= S7;
        else
          S <= S4;
        end if;
-- pic access
      when S5 => 
	    if(pic_busy = '1')then
          S <= S7;
        else
          S <= S5;
        end if;
-- bus controller access
      when S6 =>
	    if(bus_done = '1')then
          S <= S7;
        else
          S <= S6;
        end if;
        if(Sbus = '1')then
          count <= count + 1;
        end if;
-- wait for cycle end
      when S7 =>
	    if(lt_tsr(1 downto 0) = "00")then
          S <= S0;
        else
          S <= S7;
        end if;
    end case;
  end if;
end process;
-- JTAG write state machine
process(clk,S)
begin 
  if(S /= S4)then
    Sjtag <= '0';
    freq_div <= "000000";
    tclk <= '0';
    jtag_done <= '0';
  elsif ( clk'event and clk = '1')then
    if(Sjtag = '0')then
	  if(lt_dxfrn = '0')then
	    if(l_adro(2) = '0')then
          Sjtag <= '1';
          jtag_data <= l_dato;
        else
	      jtag_csr <= l_dato(14 downto 0);
          jtag_done <= '1';
        end if;
      end if;
    else
	  if(freq_div = fd)then
        freq_div <= "000000";
        tclk <= not tclk;
        if(tclk = '1')then
	      jtag_data(31 downto 16) <= jtag_data(30 downto 16) & '0'; 
	      jtag_data(15 downto 0) <= jtag_data(14 downto 0) & jtag_tdo; 
	    end if;
        if(count = 15 and tclk = '1')then
          Sjtag <= '0';
          jtag_done <= '1';
        end if;
      else
        freq_div <= freq_div + 1;
      end if;
    end if;
  end if;
end process;
-- pic write state machine
process(clk,rstn)
begin 
  if(rstn = '0')then
    Spic <= Sa0;
    pic_busy <= '0';
    timeout <= '1';
  elsif (clk'event and clk = '1')then
    if(sample = '1')then
      case Spic is
        when Sa0 =>
          if(S = S5 and lt_dxfrn = '0')then
            Spic <= Sa1;
            pic_busy <= '1';
          else
            Spic <= Sa0;
            pic_busy <= '0';
          end if;
        when Sa1 =>
          if(cycles = 9)then  
            Spic <= Sa2;
          else
            Spic <= Sa1;
          end if;
          timeout <= '0';
        when Sa2 =>
          if(cycles = 12)then
            Spic <= Sa0;
            timeout <= '1';
          elsif(ack = '1' and most_zero = '1')then  
            Spic <= Sa3;
          else
            Spic <= Sa2;
          end if;
        when Sa3 =>
          if(cycles = 11)then  
            Spic <= Sa0;
          else
            Spic <= Sa3;
          end if;
      end case;
    end if;
  end if;
end process;
-- bus controller state machine
process(clk,S)
begin 
  if(S /= S6)then
    Sbus <= '0';
    bus_done <= '0';
    write_bus <= '0';
  elsif ( clk'event and clk = '1')then
    if(Sbus = '0')then
      bus_done <= '0';
      if(lt_dxfrn = '0' or l_cmdo(0) = '0')then
        write_bus <= l_cmdo(0);
        Sbus <= '1';
      end if;
    else
      if(count = 15)then
        Sbus <= '0';
        bus_done <= '1';
      end if;
    end if;
  end if;
end process;
-- Active JTAG port selection logic 
-- jtag_csr(10 downto 9) PCI bus number
-- jtag_csr(8 downto 6) PCI device number
process(jtag_csr,jtag_data,tclk,tdo)
variable tdi_mux,tms_mux: std_logic;
begin
  case jtag_csr(14 downto 11) is
    when "0000" =>
    		    tdi_mux := jtag_data(0);
    		    tms_mux := jtag_data(16);
    when "0001" =>
    		    tdi_mux := jtag_data(1);
    		    tms_mux := jtag_data(17);
    when "0010" =>
    		    tdi_mux := jtag_data(2);
    		    tms_mux := jtag_data(18);
    when "0011" =>
    		    tdi_mux := jtag_data(3);
    		    tms_mux := jtag_data(19);
    when "0100" =>
    		    tdi_mux := jtag_data(4);
    		    tms_mux := jtag_data(20);
    when "0101" =>
    		    tdi_mux := jtag_data(5);
    		    tms_mux := jtag_data(21);
    when "0110" =>
    		    tdi_mux := jtag_data(6);
    		    tms_mux := jtag_data(22);
    when "0111" =>
    		    tdi_mux := jtag_data(7);
    		    tms_mux := jtag_data(23);
    when "1000" =>
    		    tdi_mux := jtag_data(8);
    		    tms_mux := jtag_data(24);
    when "1001" =>
    		    tdi_mux := jtag_data(9);
    		    tms_mux := jtag_data(25);
    when "1010" =>
    		    tdi_mux := jtag_data(10);
    		    tms_mux := jtag_data(26);
    when "1011" =>
    		    tdi_mux := jtag_data(11);
    		    tms_mux := jtag_data(27);
    when "1100" =>
    		    tdi_mux := jtag_data(12);
    		    tms_mux := jtag_data(28);
    when "1101" =>
    		    tdi_mux := jtag_data(13);
    		    tms_mux := jtag_data(29);
    when "1110" =>
    		    tdi_mux := jtag_data(14);
    		    tms_mux := jtag_data(30);
    when others =>
    		    tdi_mux := jtag_data(15);
    		    tms_mux := jtag_data(31);
  end case;
  tdi <= (others => 'Z');
  tck <= (others => 'Z');
  tms <= (others => 'Z');
  case jtag_csr(10 downto 6) is
    when "00001" => jtag_tdo <= tdo(9); -- log3
    		    tdi(9) <= tdi_mux;
    		    tms(9) <= tms_mux;
    		    tck(9) <= tclk;
    when "00011" => jtag_tdo <= tdo(8); -- bc
    		    tdi(8) <= tdi_mux;
    		    tms(8) <= tms_mux;
    		    tck(8) <= tclk;
    when "01100" => jtag_tdo <= tdo(7); -- mip6
    		    tdi(7) <= tdi_mux;
    		    tms(7) <= tms_mux;
    		    tck(7) <= tclk;
    when "01011" => jtag_tdo <= tdo(6); -- mip5
    		    tdi(6) <= tdi_mux;
    		    tms(6) <= tms_mux;
    		    tck(6) <= tclk;
    when "01010" => jtag_tdo <= tdo(5); -- mip4
    		    tdi(5) <= tdi_mux;
    		    tms(5) <= tms_mux;
    		    tck(5) <= tclk;
    when "01001" => jtag_tdo <= tdo(4); -- log2
    		    tdi(4) <= tdi_mux;
    		    tms(4) <= tms_mux;
    		    tck(4) <= tclk;
    when "10100" => jtag_tdo <= tdo(3); -- mip3
    		    tdi(3) <= tdi_mux;
    		    tms(3) <= tms_mux;
    		    tck(3) <= tclk;
    when "10011" => jtag_tdo <= tdo(2); -- mip2
    		    tdi(2) <= tdi_mux;
    		    tms(2) <= tms_mux;
    		    tck(2) <= tclk;
    when "10010" => jtag_tdo <= tdo(1); -- mip1
    		    tdi(1) <= tdi_mux;
    		    tms(1) <= tms_mux;
    		    tck(1) <= tclk;
    when "10001" => jtag_tdo <= tdo(0); -- log1
    		    tdi(0) <= tdi_mux;
    		    tms(0) <= tms_mux;
    		    tck(0) <= tclk;
    when others => jtag_tdo <= '0';
  end case;
end process;
-- FLASH read/write control
process(S,lt_tsr)
begin
  case S is
    when S1 =>
      flash_cen <= not lt_tsr(0);
      flash_oen <= not lt_tsr(0);
    when S3 =>
      flash_cen <= '0';
      flash_oen <= '1';
    when others =>
      flash_cen <= '1';
      flash_oen <= '1';
  end case;
end process;       
-- lt_rdyn and lt_discn
process(S,lt_dxfrn,pic_retry,l_cmdo,count)
begin
  case S is
    when S1 =>
      if(count = 3)then
        lt_rdyn <= '0';
        lt_discn <= lt_dxfrn;
      else
        lt_rdyn <= '1';
        lt_discn <= '1';
      end if;
    when S2 | S4 | S6 =>
      lt_rdyn <= not l_cmdo(0);
      lt_discn <= '0';
    when S5 =>
      lt_rdyn <= pic_retry;
      lt_discn <= '0';
    when S3 | S7 =>
      lt_rdyn <= '1';
      lt_discn <= '0';
    when others => 
      lt_rdyn <= '1';
      lt_discn <= '1';
  end case;
end process;
-- bus controller interface
process(clk)
begin
  if(clk'event and clk = '1')then
    if(Sbus = '0')then
      j3bus_ctl1p <= '0';
      t_j3bus_ctl1 <= bus_done;
    else
      case count is
        when "0001" =>
          j3bus_ctl1p <= '0';
        when "0010" =>
          j3bus_ctl1p <= '1';
        when "0011" =>
          j3bus_ctl1p <= '0';
        when "0100" =>
          j3bus_ctl1p <= temp(1) and write_bus;
        when "0101" =>
          j3bus_ctl1p <= temp(0);
        when others =>
          j3bus_ctl1p <= '1';
      end case;
      case count is
        when "0110" | "0111" | "1000" | "1001" | "1010" | "1011" | "1100" | "1101" | "1110" | "1111" =>
          t_j3bus_ctl1 <= '1';
        when others =>
          t_j3bus_ctl1 <= '0';
      end case;
      if(count(3) = '1')then
        if(count = 15)then
          if(bus_data(6 downto 3) /= "1010")then
            bus_data <= "11111111";
          else
            bus_data <= "0000" & bus_data(2 downto 0) & j3bus_ctl1;
          end if;
        else
          bus_data <= bus_data(6 downto 0) & j3bus_ctl1;
        end if;
      end if;
    end if;
  end if;
end process;
j3bus_ctl1 <= j3bus_ctl1p when t_j3bus_ctl1 = '0' else 'Z';
j3bus_ctl0 <= clk;
-- baud rate generator
process(clk,Spic)
begin
  if(Spic = Sa0)then
    diva <= 0;
    sample <= '1';
    div16 <= 0;
    cycles <= 0;
  elsif(clk'event and clk = '1')then
    if(diva = tc208)then
      diva <= 0;
      sample <= '1';
    else
      diva <= diva + 1;
      sample <= '0';
    end if;
    if(sample = '1')then
      if(Spic = Sa2 and ack = '1' and most_zero = '1')then
        div16 <= 8;
        cycles <= 0;
      else
        div16 <= div16 + 1;
        if(div16 = 15)then
          cycles <= cycles + 1;
        end if;
      end if;
    end if;
  end if;
end process;
-- serial/deserializer
serial <= '0' when pic_data(8) = '0' else 'Z';
process(clk)
begin
  if(clk'event and clk = '1')then
    if(sample = '1')then
      serial_in <= serial_in(2 downto 0) & serial;
      most_one <= (serial_in(3) and serial_in(2) and serial_in(1)) or
                  (serial_in(3) and serial_in(2) and serial_in(0)) or
                  (serial_in(3) and serial_in(1) and serial_in(0)) or
                  (serial_in(2) and serial_in(1) and serial_in(0));
      most_zero <= not(serial_in(3) or serial_in(2) or serial_in(1)) or
                   not(serial_in(3) or serial_in(2) or serial_in(0)) or
                   not(serial_in(3) or serial_in(1) or serial_in(0)) or
                   not(serial_in(2) or serial_in(1) or serial_in(0));
      ack <= most_one or (ack and not most_zero);
    end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then 
    if(Spic = Sa0)then
      if(S = S5 and lt_dxfrn = '0')then
        pic_data(7 downto 0) <= l_dato(7 downto 0);
        pic_data(8) <= '1';
      else
        pic_data(8) <= '0';
      end if;  
    elsif(sample = '1' and div16 = 15)then
      if(Spic = Sa1)then
        pic_data(8) <= pic_data(7);
      else
        pic_data(8) <= '1';
      end if;
      if(cycles < 9)then
        pic_data(7 downto 0) <= pic_data(6 downto 0) & ack;
      end if;
    end if; 
  end if;
end process;
end architecture logic_design;   --End of architecture block
